@@ -65,7 +65,7 @@ $_ResetEvent = new Win32::API("kernel32", "ResetEvent", [N], I);
 use strict;
 
 use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $RBUF_Size);
-$VERSION = '0.16';
+$VERSION = '0.17';
 $RBUF_Size = 4096;
 
 require Exporter;
@@ -519,6 +519,8 @@ my $zero=0;
 
 sub OS_Error { print Win32::FormatMessage ( Win32::GetLastError() ); }
 
+sub get_tick_count { return Win32::GetTickCount(); }
+
     # test*.t only - suppresses default messages
 sub set_no_messages {
     return unless (@_ == 2);
@@ -2443,11 +2445,11 @@ Win32API::CommPort - Raw Win32 system API calls for serial communications.
 
 =head1 SYNOPSIS
 
-  use Win32;
+  use Win32;	## not required under all circumstances
   require 5.003;
-  use Win32API::CommPort qw( :PARAM :STAT 0.16 );
+  use Win32API::CommPort qw( :PARAM :STAT 0.17 );
 
-  ## when available ##  use Win32API::File 0.05 qw( :ALL );
+  ## when available ##  use Win32API::File 0.06 qw( :ALL );
 
 =head2 Constructors
 
@@ -2474,6 +2476,8 @@ Win32API::CommPort - Raw Win32 system API calls for serial communications.
   $PortObj->fetch_DCB  || die "Not done";
   $PortObj->update_DCB || die "Not done";
 
+  $milliseconds = $PortObj->get_tick_count;
+
 =head2 Capability Methods (read only)
 
      # true/false capabilities
@@ -2761,6 +2765,11 @@ I<Win32::SerialPort>. Please let me know if one does not work as advertised.
   $PortObj->tx_on_xoff(0);	# continue output even after input xoff sent
 				# [fTXContinueOnXoff]
 
+The B<get_tick_count> method is a wrapper around the I<Win32::GetTickCount()>
+function. It matches a corresponding method in I<Device::SerialPort> which
+does not have access to the I<Win32::> namespace. It still returns time
+in milliseconds - but can be used in cross-platform scripts.
+
 =head2 Configuration and Capability Methods
 
 The Win32 Serial Comm API provides extensive information concerning
@@ -3136,6 +3145,6 @@ ActiveState 3xx builds. Users of those builds will need to install
 differently (see README). Programs in the test suite are modified for
 the current version. Additions to the configurtion files generated by
 B<save> prevent those created by Version 0.15 from being used by earlier
-Versions. 18 July 1999.
+Versions. 12 August 1999.
 
 =cut
